import math
import numpy as np
import os
import sys

sys.path.insert(0, '../../')

import utility.generator as generator
import algorithms.chen_thm_4_4 as chen4
import algorithms.chen_thm_4_6 as chen6
import algorithms.chen_thm_4_7 as chen7
import algorithms.baker as baker
import algorithms.baruah as load
import algorithms.huang as huang

def store(results):
    name = str(results['id']) + '_' + str(results['scheme']) + '_' + str(results['processors']) + '_' + str(results['population']) + '_' + '_'.join(str(e) for e in results['period']) + '_' + '_'.join(str(e) for e in results['deadline']) + '_'
    np.save(os.path.join('./results', name), results)

def main():
    # number of processors
    # eg., processors = [4, 8, 16, 32]
    processors = [8]
    
    # FP-Schema 
    # e.g., schemes = ['DM', 'SM', 'RM']
    schemes = ['DM']
    
    # range of periods (uniform) -- True if periods are integers
    # e.g., periods = [(min, max, True), (min, max, False),...]
    periods = [(1, 10, False), (1, 100, False), (1, 1000, False)]
    
    # deadline to period ratios (uniform) -- min <= x <= max, Di = x * Ti
    # e.g., deadlines = [(min, max), (min, max), .., (min, max)]
    deadlines = [(0.8, 2)]
    
    # difference bewteen two utilization values 
    steps = 0.05 

    # number of tasks per task set
    # e.g., populations = [20, 40, 50]
    populations = [40]
    
    # number of task sets
    samples = 100

    for processor in processors:
        for scheme in schemes:
            for population in populations: 
                for deadline in deadlines:        
                    for period in periods:
                        fails_CH4 = []
                        fails_CH6 = []
                        fails_CH7 = []
                        fails_LOAD = []
                        fails_BAK = []
                        fails_HC = []
                        fails_ALL = []

                        for utilization in np.arange(steps, 1.0 + steps, steps):
                            tasksets = generator.generate(population, samples, int(processor)*utilization, period[0], period[1], period[2], deadline[0], deadline[1])
                            fails_LOAD.append(0)
                            fails_BAK.append(0)
                            fails_HC.append(0)
                            fails_CH4.append(0)
                            fails_CH6.append(0)
                            fails_CH7.append(0)
                            fails_ALL.append(0)

                            for taskset in tasksets:
                                if scheme == 'RM':
                                    taskset = sorted(taskset, key = lambda task : task['period'])

                                if scheme == 'SM':
                                    taskset = sorted(taskset, key = lambda task : task['deadline'] - task['execution'])

                                if scheme == 'DM':
                                    taskset = sorted(taskset, key = lambda task : task['deadline'])
                                            
                                load_eval = True
                                baker_eval = True
                                huang_eval = True
                                chen4_eval = True
                                chen6_eval = True
                                chen7_eval = True

                                if load.schedulable(taskset, processor) is False:
                                    load_eval = False
                                    fails_LOAD[-1] += 1

                                if baker.schedulable(taskset, processor) is False:
                                    baker_eval = False
                                    fails_BAK[-1] += 1
                                    
                                if huang.schedulable(taskset, processor) is False:
                                    huang_eval = False
                                    fails_HC[-1] += 1
                                    
                                if chen4.schedulable(taskset, processor) is False:
                                    chen4_eval = False
                                    fails_CH4[-1] += 1

                                if chen6.schedulable(taskset, processor) is False:
                                    chen6_eval = False
                                    fails_CH6[-1] += 1
                                    
                                if chen7.schedulable(taskset, processor) is False:
                                    chen7_eval = False
                                    fails_CH7[-1] += 1
                                
                                if (not load_eval) and (not baker_eval) and (not huang_eval) and (not chen4_eval) and (not chen6_eval) and (not chen7_eval):
                                    fails_ALL[-1] += 1
                        
                        baker_results = {'id' : 'BAK', 'fails' : fails_BAK, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}
                        
                        load_results = {'id' : 'LOAD', 'fails' : fails_LOAD, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        huang_results = {'id' : 'HC', 'fails' : fails_HC, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        chen4_results = {'id' : 'OUR-4.4', 'fails' : fails_CH4, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        chen6_results = {'id' : 'OUR-4.6', 'fails' : fails_CH6, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        chen7_results = {'id' : 'OUR-4.7', 'fails' : fails_CH7, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        all_results = {'id' : 'ALL', 'fails' : fails_ALL, 'population' : population, 
                        'scheme' : scheme, 'processors' : processor, 'period' : period, 'deadline' : deadline, 'steps' : steps, 'samples' : samples}

                        store(baker_results)
                        store(load_results)
                        store(huang_results)
                        store(chen4_results)
                        store(chen6_results)
                        store(chen7_results)
                        store(all_results)

if __name__ == "__main__":
    main()
