from __future__ import division
import sys
import numpy as np
import matplotlib.pyplot as plt
import itertools
from matplotlib import rcParams

rcParams['font.family'] = 'sans-serif'
rcParams['font.sans-serif'] = ['Tahoma']
rcParams['ps.useafm'] = True
rcParams['pdf.use14corefonts'] = True
rcParams['text.usetex'] = True

def plot (title, filenames):
    figlabel = itertools.cycle(('a','b','c','d','e','f','g','h','i'))
    marker = itertools.cycle(('v', 's', '*', 'D', 'x', '+', 'o'))
    colors = itertools.cycle(('c', 'g', 'b', 'y', 'm', 'k', 'r'))
    names = []
    fig = plt.figure()
    ax = fig.add_subplot(111)
    fig.subplots_adjust (top = 0.9, left = 0.1, right = 0.95, hspace = 0.3)
    ax.set_xlabel('Utilization/M (\%)',size=12)
    ax.set_ylabel('Acceptance Ratio (\%)',size=12)
    ax.spines['top'].set_color('none')
    ax.spines['bottom'].set_color('none')
    ax.spines['left'].set_color('none')
    ax.spines['right'].set_color('none')
    ax.tick_params(labelcolor = 'w', top = 'off', bottom = 'off', left = 'off', right = 'off')
    
    for filename in filenames:
        data = np.load(filename)
        data = data.item()
        utilization = list(100*np.arange(data['steps'], 1.0 + data['steps'], data['steps']))
        acceptance = map(lambda tick : 100*((data['samples'] - tick)/(data['samples'])), data['fails'])        
        ax.axis([-2,102,-2,102])
        ax.plot(utilization, acceptance, '-', color = colors.next(), 
        marker = marker.next(), markersize = 7, markevery = 1, fillstyle = 'full', label = data['id'], linewidth = 1.5)
        names.append (data['id'])
        ax.tick_params(labelcolor='k', top='off', bottom='off', left='off', right='off')
        
        for tick in ax.xaxis.get_major_ticks():
            tick.label.set_fontsize(10)
        
        for tick in ax.yaxis.get_major_ticks():
            tick.label.set_fontsize(10)
         
    
    plt.title(title)
    ax.legend(names)
    ax.grid()
    return fig